@@ -113,7 +113,7 @@ namespace std {
 
       // NB: This is an unsigned type, and thus represents the maximum
       // size that the allocator can hold.
-      static const size_type npos = -1;
+      static const size_type npos = static_cast<size_type>(-1);
 
     private:
       // _Rep: string representation
@@ -478,11 +478,10 @@ namespace std {
       basic_string& 
       append(const _CharT* __s, size_type __n)
       {
-	size_type __size =  min(traits_type::length(__s), __n);
-	size_type __len = __size + this->size();
+	size_type __len = __n + this->size();
 	if (__len > this->capacity())
 	  this->reserve(__len); 
-	return this->replace(_M_iend(), _M_iend(), __s, __s + __size); 
+	return this->replace(_M_iend(), _M_iend(), __s, __s + __n); 
       }
 
       basic_string& 
@@ -679,19 +678,39 @@ namespace std {
         _M_replace(iterator __i1, iterator __i2, _FwdIterator __j1, 
 		   _FwdIterator __j2, forward_iterator_tag);
 
+      // _S_construct_aux is used to implement the 21.3.1 para 15 which
+      // requires special behaviour if _InIter is an integral type
+      template<class _InIter>
+        static _CharT*
+        _S_construct_aux(_InIter __beg, _InIter __end, const _Alloc& __a,
+			 __false_type)
+	{
+          typedef typename iterator_traits<_InIter>::iterator_category _Tag;
+          return _S_construct(__beg, __end, __a, _Tag());
+	}
+ 
+      template<class _InIter>
+        static _CharT*
+        _S_construct_aux(_InIter __beg, _InIter __end, const _Alloc& __a,
+			 __true_type)
+	{
+	  return _S_construct(static_cast<size_type>(__beg),
+			      static_cast<value_type>(__end), __a);
+	}
+ 
       template<class _InIter>
         static _CharT*
         _S_construct(_InIter __beg, _InIter __end, const _Alloc& __a)
-        { 
-	  typedef typename iterator_traits<_InIter>::iterator_category _Tag;
-	  return _S_construct(__beg, __end, __a, _Tag()); 
-	}
+	{
+	  typedef typename _Is_integer<_InIter>::_Integral _Integral;
+	  return _S_construct_aux(__beg, __end, __a, _Integral());
+        }
 
       // For Input Iterators, used in istreambuf_iterators, etc.
-       template<class _InIter>
+      template<class _InIter>
         static _CharT*
-        _S_construct(_InIter __beg, _InIter __end, const _Alloc& __a,
-		     input_iterator_tag);
+         _S_construct(_InIter __beg, _InIter __end, const _Alloc& __a,
+		      input_iterator_tag);
       
       // For forward_iterators up to random_access_iterators, used for
       // string::iterator, _CharT*, etc.
